@@ -50,33 +50,36 @@ const vfs = require('vinyl-fs');
 
 const paths = {
   ts: {
-    release: [
-      "lib/**/*.{ts,tsx}",
-      "background/*.{ts,tsx}",
-      "content_scripts/*.{ts,tsx}",
-      "popup/*.{ts,tsx}",
-      "pages/*.{ts,tsx}",
-      "!**/*.d.ts",
+    src: [
+      "src/**/*.{ts,tsx}",
+      "!src/**/*-test*.ts",
     ],
-    dev: [
-        "test/tests/*.{ts,tsx}",
+    decl: [
+      "decl/lib.es6.d.ts",
+      "decl/urijs/URIjs.d.ts",
+      "decl/systemjs/systemjs.d.ts",
+      "decl/react-global.d.ts",
+      "decl/chrome/chrome.d.ts",
+    ],
+    test: [
+        "testlib/**/.ts",
+        "src/**/*-test*.ts",
     ],
   },
+  // distributed in the chrome extension
   dist: [
-    "i18n/*.po",
-    "i18n/*.pot",
-    "img/*",
-    "style/*.css",
-    "lib/vendor/*",
-    "i18n/strings.js",
-    "lib/emscripten/libwrapper.js",
-    "lib/module-trampoline.js",
-    "popup/**/*.{html,css}",
-    "pages/**/*.{html,css}",
-    "lib/**/*.d.ts",
-    "background/*.html",
+    "img/icon.png",
+    "img/logo.png",
+    "src/**/*.{js,css,html}",
+    "testlib/**/*.{js,ts,tsx,html}",
   ],
+  // for the source distribution
   extra: [
+      "scripts/prove-node",
+      "scripts/prove-selenium",
+      "src/i18n/*.po",
+      "src/i18n/*.pot",
+      "decl/**/*.d.ts",
       "AUTHORS",
       "README",
       "COPYING",
@@ -92,37 +95,13 @@ const paths = {
       // targets other than "srcdist".
       "manifest.json",
   ],
-  /* French copyright application */
-  appdist: [
-      "i18n/*.po",
-      "i18n/*.pot",
-      "style/*.css",
-      "img/**",
-      "lib/**/*.{ts,tsx}",
-      "!lib/vendor/*",
-      "!**/*.d.ts",
-      "background/*.{ts,tsx}",
-      "content_scripts/*.{ts,tsx}",
-      "popup/*.{ts,tsx}",
-      "pages/*.{ts,tsx}",
-      "AUTHORS",
-      "README",
-      "COPYING",
-      "Makefile",
-      "gulpfile.js",
-      "test/tests/*.{ts,tsx}",
-      "pogen/pogen.ts",
-      "lib/module-trampoline.js",
-      "popup/**/*.{html,css}",
-      "pages/**/*.{html,css}",
-  ],
 };
 
 
 const tsBaseArgs = {
   target: "es6",
   jsx: "react",
-  reactNamespace: "preact",
+  reactNamespace: "React",
   experimentalDecorators: true,
   module: "system",
   sourceMap: true,
@@ -134,37 +113,7 @@ const tsBaseArgs = {
 };
 
 
-let manifest;
-(() => {
-  const f = fs.readFileSync("manifest.json", "utf8");
-  manifest = JSON.parse(f);
-})();
-
-
-/**
- * File globbing that works just like
- * gulp.src(...).
- */
-function gglob(ps) {
-  let patPos = [];
-  let patNeg = [];
-  for (let x of ps) {
-    if (x.slice(0,1) === "!") {
-      patNeg.push(x.slice(1));
-    } else {
-      patPos.push(x);
-      console.log("Pattern", x);
-   }
-  }
-  let result = new Set();
-  for (let pat of patPos) {
-    let rs = glob.sync(pat, {ignore: patNeg});
-    for (let r of rs) {
-      result.add(r);
-    }
-  }
-  return Array.from(result);
-}
+const manifest = JSON.parse(fs.readFileSync("manifest.json", "utf8"));
 
 
 // Concatenate node streams,
@@ -172,7 +121,7 @@ function gglob(ps) {
 function concatStreams (/*streams...*/) {
   var toMerge = [].slice.call(arguments);
   if (toMerge.length === 1 && (toMerge[0] instanceof Array)) {
-    toMerge = toMerge[0]; //handle array as arguments object
+    toMerge = toMerge[0]; // handle array as arguments object
   }
   var stream = new Stream();
   stream.setMaxListeners(0); // allow adding more than 11 streams
@@ -220,7 +169,12 @@ gulp.task("compile-prod", ["clean"], function () {
   tsArgs.outDir = ".";
   // We don't want source maps for production
   tsArgs.sourceMap = undefined;
-  return gulp.src(paths.ts.release, {base: "."})
+  let opts = {base: "."};
+  const files = concatStreams(
+          gulp.src(paths.ts.src, opts),
+          gulp.src(paths.ts.decl, opts));
+
+  return files
       .pipe(ts(tsArgs))
       .pipe(gulp.dest("build/ext/"));
 });
@@ -228,7 +182,7 @@ gulp.task("compile-prod", ["clean"], function () {
 gulp.task("manifest-stable", ["clean"], function () {
   return gulp.src("manifest.json")
              .pipe(jsonTransform((data) => {
-               data.name = "GNU Taler Wallet (stable)";
+               data.name = "GNU Taler Wallet";
                return data;
              }, 2))
              .pipe(gulp.dest("build/ext/"));
@@ -273,8 +227,9 @@ gulp.task("srcdist", [], function () {
   const opts = {buffer: false, stripBOM: false, base: "."};
   // We can't just concat patterns due to exclude patterns
   const files = concatStreams(
-      gulp.src(paths.ts.release, opts),
-      gulp.src(paths.ts.dev, opts),
+      gulp.src(paths.ts.src, opts),
+      gulp.src(paths.ts.decl, opts),
+      gulp.src(paths.ts.test, opts),
       gulp.src(paths.dist, opts),
       gulp.src(paths.extra, opts));
 
@@ -286,18 +241,6 @@ gulp.task("srcdist", [], function () {
 });
 
 
-/**
- * Create source distribution for
- * French copyright application.
- */
-gulp.task("appdist", [], function () {
-  let zipname = String.prototype.concat("taler-wallet-webex-", manifest.version_name, "-appsrc.zip");
-  return gulp.src(paths.appdist, {buffer: false, stripBOM: false, base: "."})
-             .pipe(zip(zipname))
-             .pipe(gulp.dest("."));
-});
-
-
 /**
  * Compile po extraction script.
  */
@@ -332,6 +275,7 @@ function tsconfig(confBase) {
     conf.files.push(file.relative);
     cb();
   }, function(cb) {
+    conf.files.sort();
     let x = JSON.stringify(conf, null, 2);
     let f = new File({
       path: "tsconfig.json",
@@ -346,9 +290,13 @@ function tsconfig(confBase) {
 // Generate the tsconfig file
 // that should be used during development.
 gulp.task("tsconfig", function() {
-  return gulp.src(Array.prototype.concat(paths.ts.release, paths.ts.dev), {base: "."})
-             .pipe(tsconfig(tsBaseArgs))
-             .pipe(gulp.dest("."));
+  let opts = {base: "."};
+  const files = concatStreams(
+          vfs.src(paths.ts.src, opts),
+          vfs.src(paths.ts.test, opts),
+          vfs.src(paths.ts.decl, opts));
+  return files.pipe(tsconfig(tsBaseArgs))
+              .pipe(gulp.dest("."));
 });
 
 
